﻿<#	
	.SYNOPSIS
        Get events of all logs.

	.DESCRIPTION
        Get events of all logs on your local machine.
        
        Events of the 'PowerShell'-Eventlog-Providers are not listed!
            
        =================================================================
        Warning:
        =================================================================
        Be careful with the timespan between $StartTime and $EndTime because of performance issues!

	.PARAMETER Levels 
        Valid Values:
            0	Undefined   Indicates logs at all levels.
            1	Critical    Indicates logs for a critical alert.
            2	Error       Indicates logs for an error.
            3	Warning     Indicates logs for a warning.
            4	Information Indicates logs for an informational message.
            5	Verbose     Indicates logs at all levels.
        
        Default = 1 , 2 , 3
	
    .PARAMETER StartTime 
        Default = ( Get-Date ) - ( New-TimeSpan -Hours 1 )
	
    .PARAMETER EndTime
        Default = ( Get-Date ) 

    .PARAMETER GroupMessages
        Group identical messages for better overview.

	.EXAMPLE
        PS C:\> Get-EventViewerLog | Format-Table -AutoSize -Wrap | Out-String -Width 4096
        

        List all 'critcal', 'error' and 'warning' events (=default) which occurred in the last hour (=default), 
        and show the results as string in an autosized and wrapped table.

    .EXAMPLE
        PS C:\> Get-EventViewerLog -Levels (1, 2, 3) -StartTime (( Get-Date ) - ( New-TimeSpan -Hours 1 )) -EndTime (Get-Date) -GroupMessages | Out-GridView


        List all 'critcal', 'error' and 'warning' events which occurred in the last hour, 
        group it by messages and show the results in a gridview.

    .NOTES
        ===============================================================================	
        # Created by: DPXone (2016-04-15)
        ===============================================================================	
#> 

Function Get-EventViewerLog { 
	# Created by: DPXone (2016-04-15)       
	Param ( 
		[ ValidateRange( 0 , 5 ) ] 
		[ Alias( 'L' ) ] 
		[ uint16[] ] $Levels = @( 1 , 2 , 3 ) , 
		[ Alias( 'ST' ) ] 
		[ datetime ] $StartTime = ( Get-Date ) - ( New-TimeSpan -Hours 1 ) , 
		[ Alias( 'ET' ) ] 
		[ datetime ] $EndTime = ( Get-Date ) , 
		[ Alias( 'G' ) ] 
		[ switch ] $GroupMessages = $false 
	) 
	Begin { 
		$Events = @() 
		$objCol = @() 
	} 
	Process { 
		$Logs = Get-WinEvent -ListLog * -ErrorAction Ignore 
		
		Foreach ( $Log In $Logs ) { 
			$hash = @{ LogName = $Log.LogName ; Level = $Levels ; StartTime = $StartTime ; EndTime = $EndTime } 
			$temp = Get-WinEvent -FilterHashtable $hash -ErrorAction SilentlyContinue | ? { $_.ProviderName -notlike '*PowerShell*' } 
			If ( $temp ) { 
				$Events += $temp 
			} 
			Clear-Variable temp -Force 
		} 
		
		If ( $GroupMessages ) { 
			Foreach ( $Group In ( $Events | Group-Object Message ) ) { 
				$obj = New-Object psobject $Group.Group[ 0 ] 
				$obj | Add-Member NoteProperty -Name 'Count' -value $Group.count 
				$objCol += $obj 
			} 
			$Events = $objCol 
		} 
	} 
	End { 
		$Events | sort TimeCreated -d | Select TimeCreated , LevelDisplayName , ProviderName , ID , TaskDisplayName , Message , Count 
	} 
} 